@@ -1,4 +1,4 @@
-# last modified 7 August 2006 by J. Fox
+# last modified 22 May 2007 by J. Fox
 
 # Data menu dialogs
 
@@ -1646,4 +1646,29 @@ Stack <- function(){
     tkgrid(buttonsFrame, sticky="w", columnspan=2)
     dialogSuffix(rows=5, columns=2, preventGrabFocus=TRUE)
     }
+    
+loadDataSet <- function() {
+    file <- tclvalue(tkgetOpenFile(filetypes=
+        gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA"}} {"All Files" {"*"}}')))
+    if (file == "") {
+        if (getRcmdr("grab.focus")) tkgrab.release(top)
+        tkdestroy(top)
+        return()
+        }
+    command <- paste('load("', file,'")', sep="")
+    dsname <- justDoIt(command)
+    logger(command)
+    activeDataSet(dsname)
+    tkfocus(CommanderWindow())
+    }
+    
+saveDataSet <- function() {
+    file <- tclvalue(tkgetSaveFile(filetypes=
+        gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA"}} {"All Files" {"*"}}'),
+        defaultextension="rda", initialfile=paste(activeDataSet(), "rda", sep=".")))
+    if (file == "") return()
+    command <- paste('save("', activeDataSet(), '", file="', file, '")', sep="")
+    justDoIt(command)
+    logger(command)
+    }
 
